# coding=utf-8

import os
from earthdiagnostics.diagnostic import Diagnostic
from earthdiagnostics.utils import Utils, TempFile
from earthdiagnostics.variable import Domain


class MonthlyMean(Diagnostic):
    """
    Calculates monthly mean for a given variable

    :original author: Javier Vegas-Regidor<javier.vegas@bsc.es>

    :created: July 2016

    :param data_manager: data management object
    :type data_manager: DataManager
    :param startdate: startdate
    :type startdate: str
    :param member: member number
    :type member: int
    :param chunk: chunk's number
    :type chunk: int
    :param variable: variable's name
    :type variable: str
    :param domain: variable's domain
    :type domain: Domain
    :param frequency: original frequency
    :type frequency: str
    :param grid: original data grid
    :type grid: str
    """

    alias = 'monmean'
    "Diagnostic alias for the configuration file"

    def __init__(self, data_manager, startdate, member, chunk, domain, variable, frequency, grid):
        Diagnostic.__init__(self, data_manager)
        self.startdate = startdate
        self.member = member
        self.chunk = chunk
        self.variable = variable
        self.domain = domain
        self.frequency = frequency
        self.grid = grid

    def __str__(self):
        return 'Calculate monthly mean Startdate: {0} Member: {1} Chunk: {2} ' \
               'Variable: {3}:{4}'.format(self.startdate, self.member, self.chunk, self.domain, self.variable)

    def __eq__(self, other):
        return self.startdate == other.startdate and self.member == other.member and self.chunk == other.chunk and \
            self.domain == other.domain and self.variable == other.variable and self.frequency == other.frequency and \
            self.grid == other.grid

    @classmethod
    def generate_jobs(cls, diags, options):
        """
            Creates a job for each chunk to compute the diagnostic

            :param diags: Diagnostics manager class
            :type diags: Diags
            :param options: variable, domain, frequency=day, grid=''
            :type options: list[str]
            :return:
            """
        num_options = len(options) - 1
        if num_options < 2:
            raise Exception('You must specify the variable and domain to average monthly')
        if num_options > 4:
            raise Exception('You must specify between 2 and 4 parameters for the monthly mean diagnostic')
        variable = options[1]
        domain = Domain(options[2])
        if num_options >= 3:
            frequency = options[3]
        else:
            frequency = 'day'
        if num_options >= 4:
            grid = options[4]
        else:
            grid = None

        job_list = list()
        for startdate, member, chunk in diags.config.experiment.get_chunk_list():
            job_list.append(MonthlyMean(diags.data_manager, startdate, member, chunk, domain, variable,
                                        frequency, grid))
        return job_list

    def compute(self):
        """
        Runs the diagnostic
        """
        temp = TempFile.get()
        variable_file = self.data_manager.get_file(self.domain, self.variable, self.startdate, self.member, self.chunk,
                                                   frequency=self.frequency, grid=self.grid)
        Utils.cdo.monmean(input=variable_file, output=temp, options='-O')
        os.remove(variable_file)

        self.send_file(temp, self.domain, self.variable, self.startdate, self.member, self.chunk,
                       frequency='mon', grid=self.grid)

