# coding=utf-8
from earthdiagnostics.diagnostic import Diagnostic
from earthdiagnostics.variable import Domain


class Rewrite(Diagnostic):
    """
    Rewrites files without doing any calculations.
    Can be useful to convert units or to correct wrong metadata

    :original author: Javier Vegas-Regidor<javier.vegas@bsc.es>

    :created: July 2016

    :param data_manager: data management object
    :type data_manager: DataManager
    :param startdate: startdate
    :type startdate: str
    :param member: member number
    :type member: int
    :param chunk: chunk's number
    :type chunk: int
    :param variable: variable's name
    :type variable: str
    :param domain: variable's domain
    :type domain: Domain
    """

    alias = 'rewrite'
    "Diagnostic alias for the configuration file"

    def __init__(self, data_manager, startdate, member, chunk, domain, variable, grid):
        Diagnostic.__init__(self, data_manager)
        self.startdate = startdate
        self.member = member
        self.chunk = chunk
        self.variable = variable
        self.domain = domain
        self.grid = grid

    def __str__(self):
        return 'Rewrite output Startdate: {0} Member: {1} Chunk: {2} ' \
               'Variable: {3}:{4} Grid: {5}'.format(self.startdate, self.member, self.chunk, self.domain, self.variable,
                                                    self.grid)

    def __eq__(self, other):
        return self.startdate == other.startdate and self.member == other.member and self.chunk == other.chunk and \
            self.domain == other.domain and self.variable == other.variable and self.grid == self.grid

    @classmethod
    def generate_jobs(cls, diags, options):
        """
            Creates a job for each chunk to compute the diagnostic

            :param diags: Diagnostics manager class
            :type diags: Diags
            :param options: variable, domain, grid
            :type options: list[str]
            :return:
            """
        num_options = len(options) - 1
        if num_options < 2:
            raise Exception('You must specify the variable and domain to rewrite')
        if num_options > 3:
            raise Exception('You must between 2 and 3 parameters for the rewrite diagnostic')
        variable = options[1]
        domain = Domain(options[2])
        if num_options >= 3:
            grid = options[3]
        else:
            grid = None
        job_list = list()
        for startdate, member, chunk in diags.config.experiment.get_chunk_list():
            job_list.append(Rewrite(diags.data_manager, startdate, member, chunk, domain, variable, grid))
        return job_list

    def compute(self):
        """
        Runs the diagnostic
        """
        variable_file = self.data_manager.get_file(self.domain, self.variable, self.startdate, self.member, self.chunk,
                                                   grid=self.grid)
        self.send_file(variable_file, self.domain, self.variable, self.startdate, self.member, self.chunk,
                       grid=self.grid)

