# coding=utf-8
from earthdiagnostics.diagnostic import Diagnostic
from earthdiagnostics.utils import Utils
from earthdiagnostics.variable import Domain
import numpy as np


class Scale(Diagnostic):
    """
    Scales a variable by the given value also adding at offset
    Can be useful to correct units or other known errors
    (think of a tas file declaring K as units but with the data stored as Celsius)

    :original author: Javier Vegas-Regidor<javier.vegas@bsc.es>

    :created: July 2016

    :param data_manager: data management object
    :type data_manager: DataManager
    :param startdate: startdate
    :type startdate: str
    :param member: member number
    :type member: int
    :param chunk: chunk's number
    :type chunk: int    :
    :param variable: variable's name
    :type variable: str
    :param domain: variable's domain
    :type domain: Domain
    """

    alias = 'scale'
    "Diagnostic alias for the configuration file"

    def __init__(self, data_manager, startdate, member, chunk, value, offset, domain, variable, grid,
                 min_limit, max_limit):
        Diagnostic.__init__(self, data_manager)
        self.startdate = startdate
        self.member = member
        self.chunk = chunk
        self.variable = variable
        self.domain = domain
        self.grid = grid
        self.value = value
        self.offset = offset
        self.min_limit = min_limit
        self.max_limit = max_limit

        self.original_values = None

    def __str__(self):
        return 'Scale output Startdate: {0} Member: {1} Chunk: {2} ' \
               'Scale value: {5} Offset: {6} Variable: {3}:{4}'.format(self.startdate, self.member, self.chunk,
                                                                       self.domain, self.variable,
                                                                       self.value, self.offset)

    def __eq__(self, other):
        return self.startdate == other.startdate and self.member == other.member and self.chunk == other.chunk and \
            self.domain == other.domain and self.variable == other.variable

    @classmethod
    def generate_jobs(cls, diags, options):
        """
            Creates a job for each chunk to compute the diagnostic

            :param diags: Diagnostics manager class
            :type diags: Diags
            :param options: variable, domain, grid
            :type options: list[str]
            :return:
            """
        num_options = len(options) - 1
        if num_options < 4:
            raise Exception('You must specify the acale and offset values and the variable and domain to scale')
        if num_options > 5:
            raise Exception('You must between 4 and 5 parameters for the rewrite diagnostic')
        value = float(options[1])
        offset = float(options[2])
        variable = options[3]
        domain = Domain(options[4])
        if num_options >= 5:
            grid = options[5]
        else:
            grid = None
        if num_options >= 6:
            if options[6].lower() == 'none':
                min_limit = None
            else:
                min_limit = float(options[6])
        else:
            min_limit = None
        if num_options >= 7:
            if options[7].lower() == 'none':
                max_limit = None
            else:
                max_limit = float(options[7])
        else:
            max_limit = None
        job_list = list()
        for startdate, member, chunk in diags.config.experiment.get_chunk_list():
            job_list.append(Scale(diags.data_manager, startdate, member, chunk, value, offset, domain, variable, grid,
                                  min_limit, max_limit))
        return job_list

    def compute(self):
        """
        Runs the diagnostic
        """
        variable_file = self.data_manager.get_file(self.domain, self.variable, self.startdate, self.member, self.chunk,
                                                   grid=self.grid)

        handler = Utils.openCdf(variable_file)
        var_handler = handler.variables[self.variable]
        self.original_values = var_handler[:]
        if self._check_limits():
            var_handler[:] = self.original_values * self.value + self.offset
        handler.close()
        self.send_file(variable_file, self.domain, self.variable, self.startdate, self.member, self.chunk,
                       grid=self.grid)

    def _check_limits(self):
        if self.min_limit is not None and np.amin(self.original_values) < self.min_limit:
            return False
        if self.max_limit is not None and np.amax(self.original_values) > self.max_limit:
            return False
        return True


