# coding=utf-8
import os
from earthdiagnostics import cdftools
from earthdiagnostics.diagnostic import Diagnostic
from earthdiagnostics.utils import Utils, TempFile
from earthdiagnostics.variable import Domains


class MixedLayerSaltContent(Diagnostic):
    """
    Compute mixed layer salt content

    :original author: Virginie Guemas <virginie.guemas@bsc.es>
    :contributor: Javier Vegas-Regidor<javier.vegas@bsc.es>

    :created: February 2012
    :last modified: June 2016

    :param data_manager: data management object
    :type data_manager: DataManager
    :param startdate: startdate
    :type startdate: str
    :param member: member number
    :type member: int
    :param chunk: chunk's number
    :type chunk: int
    """
    alias = 'mlotstsc'
    "Diagnostic alias for the configuration file"

    def __init__(self, data_manager, startdate, member, chunk):
        Diagnostic.__init__(self, data_manager)
        self.startdate = startdate
        self.member = member
        self.chunk = chunk
        self.required_vars = ['so', 'mlotst']
        self.generated_vars = ['scvertsum']

    def __str__(self):
        return 'Mixed layer salt content Startdate: {0} Member: {1} Chunk: {2}'.format(self.startdate, self.member,
                                                                                       self.chunk)

    def __eq__(self, other):
        return self.startdate == other.startdate and self.member == other.member and self.chunk == other.chunk

    @classmethod
    def generate_jobs(cls, diags, options):
        """
        Creates a job for each chunk to compute the diagnostic

        :param diags: Diagnostics manager class
        :type diags: Diags
        :param options: None
        :type options: list[str]
        :return:
        """
        if len(options) > 1:
            raise Exception('The mixed layer salt content diagnostic has no options')
        job_list = list()
        for startdate, member, chunk in diags.config.experiment.get_chunk_list():
            job_list.append(MixedLayerSaltContent(diags.data_manager, startdate, member, chunk))
        return job_list

    def compute(self):
        """
        Runs the diagnostic
        """
        salinity_file = self.data_manager.get_file(Domains.ocean, 'so', self.startdate, self.member, self.chunk)
        mlotst_file = self.data_manager.get_file(Domains.ocean, 'mlotst', self.startdate, self.member, self.chunk)

        Utils.nco.ncks(input=mlotst_file, output=salinity_file, options='-A -v mlotst')

        temp = TempFile.get()
        cdftools.run('cdfmxlsaltc', input=salinity_file, output=temp)
        os.remove(salinity_file)

        Utils.rename_variables(temp, {'x': 'i', 'y': 'j', 'somxlsaltc': 'scvsummlotst'}, False, True)
        Utils.setminmax(temp, 'scvsummlotst')
        self.send_file(temp, Domains.ocean, 'scvsummlotst', self.startdate, self.member, self.chunk)
